<?php
/*
 * Date:	26.08.2007
 *
 * File:	index.php
 * Project:	AJAX_CHAT
 *
 * Author:	Sebastian Tschan
 * Website:	https://blueimp.net
 *
 * License:	GPL
 * 
 */

// Show all errors:
error_reporting(E_ALL);

// Path to the chat directory:
define('AJAX_CHAT_PATH', dirname($_SERVER['SCRIPT_FILENAME']));

// Include Class libraries:
require_once(AJAX_CHAT_PATH.'/lib/class/DataBase.php');
require_once(AJAX_CHAT_PATH.'/lib/class/SQLQuery.php');
require_once(AJAX_CHAT_PATH.'/lib/class/Functions.php');
require_once(AJAX_CHAT_PATH.'/lib/class/ContentType.php');
require_once(AJAX_CHAT_PATH.'/lib/class/Language.php');
require_once(AJAX_CHAT_PATH.'/lib/class/Chat.php');
require_once(AJAX_CHAT_PATH.'/lib/class/CustomChat.php');

// Functionality to display the XHTML content:
function ajaxChatDisplayContent($chat, $params) {
	// Initialize the language object to determine the user language:
	if(isset($_SESSION[$params['sessionValuePrefix'].'langCode']))
		// Use the language setting from the saved session:
		$language = new Language($params['langAvailable'], $params['langDefault'], $_SESSION[$params['sessionValuePrefix'].'langCode']);
	else {
		// Determine the language setting from the browser headers (HTTP_ACCEPT_LANGUAGE):
		$language = new Language($params['langAvailable'], $params['langDefault']);
		// Save the language setting in the session:
		$_SESSION[$params['sessionValuePrefix'].'langCode'] = $language->getLangCode();
	}
	// Include the language file:
	require_once(AJAX_CHAT_PATH.'/lib/lang/'.$language->getLangCode().'.php');

	// Include the template:
	if($chat->isLoggedIn())
		require_once(AJAX_CHAT_PATH.'/lib/template/loggedIn.php');
	else
		require_once(AJAX_CHAT_PATH.'/lib/template/loggedOut.php');
}

// phpBB integration:
define('IN_PHPBB', true);
$phpbb_root_path = AJAX_CHAT_PATH.'/../';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
include($phpbb_root_path . 'common.' . $phpEx);

// phpBB session management:
$user->session_begin();
$auth->acl($user->data);

// Include config parameters ($ajaxChatConfig):
require_once(AJAX_CHAT_PATH.'/lib/config.php');

// Initialize the chat:
$ajaxChat = new CustomChat($ajaxChatConfig);

// Auto-login phpBB forum users:
if(!isset($_REQUEST['logout']) && $user->data['user_id'] != ANONYMOUS) {
	// phpBB stores the ampersand ("&") as HTML entity ("&amp;") in usernames:
	$ajaxChat->setRequestUserName(str_replace('&amp;', '&', $user->data['username']));
}

// Initialize the chat session:
$ajaxChat->initSession();

if(isset($_REQUEST['ajax'])) {
	// Parse message requests:
	$ajaxChat->initMessageHandling();
	
	// Send chat messages and online user list in XML format:
	$ajaxChat->sendXMLMessages();
} else {
	// Set the style cookie depending on the phpBB user style:
	$ajaxChat->setStyle();

	// Display XHTML content for non-ajax requests:
	ajaxChatDisplayContent($ajaxChat, $ajaxChatConfig);
}
?>