<?php
/*
 * Date:	16.09.2007
 *
 * File:	lib/class/Chat.php
 * Project:	AJAX_CHAT
 *
 * Author:	Sebastian Tschan
 * Website:	https://blueimp.net
 *
 * License:	GPL
 * 
 */

// Ajax Chat backend logic:
class Chat {

	var $db;
	var $_params;
	var $_specialChars;
	var $_requestVars;
	var $_infoMessages;
	var $_channels;
	
	function Chat(&$params) {
		if(isset($params['dbConnection']['link'])) {
			// Use an existing DataBase link:
			$this->db = new DataBase(
				$params['dbConnection']['link']
			);
		} else {
			// Establish a connection to the database:
			$this->db = new DataBase(
				$params['dbConnection']
			);
			$this->db->connect($params['dbConnection']);
			if($this->db->error()) {
			    echo $this->db->getError();
			    die();
			}
			$this->db->select($params['dbConnection']);
			if($this->db->error()) {
			    echo $this->db->getError();
			    die();
			}
			// Unset the dbConnection array for safety purposes:
			unset($params['dbConnection']);			
		}

		$this->_params = $params;
		
		// As &apos; is not supported by IE, we use &#39; as replacement for "'":
		$this->_specialChars = array('&'=>'&amp;', '<'=>'&lt;', '>'=>'&gt;', "'"=>'&#39;', '"'=>'&quot;');
		
		$this->initRequestVars();
		
		$this->_infoMessages = array();
	}
	
	function initRequestVars() {
		$this->_requestVars = array();
		$this->_requestVars['userID']		= isset($_REQUEST['userID'])		? (int)$_REQUEST['userID']		: null;
		$this->_requestVars['userName']		= isset($_REQUEST['userName'])		? $_REQUEST['userName']			: null;
		$this->_requestVars['channelID']	= isset($_REQUEST['channelID'])		? (int)$_REQUEST['channelID']	: null;
		$this->_requestVars['channelName']	= isset($_REQUEST['channelName'])	? $_REQUEST['channelName']		: null;
		$this->_requestVars['text']			= isset($_REQUEST['text'])			? $_REQUEST['text']				: null;
		$this->_requestVars['lastID']		= isset($_REQUEST['lastID'])		? (int)$_REQUEST['lastID']		: 0;
		$this->_requestVars['logout']		= isset($_REQUEST['logout'])		? true							: false;
		$this->_requestVars['password']		= isset($_REQUEST['password'])		? $_REQUEST['password']			: null;
	}

	function initSession($startSession=true) {
		if($startSession) {
			// Set the session name:
			session_name($this->_params['sessionName']);
			// Start the login session (if not already  started):
			@session_start();
		}
		
		// This enables logout if JavaScript is disabled on client side:
		if($this->_requestVars['logout']) {
			$this->logout(true);
			return;
		}
		
		// If not logged in and a userName is given, try to login:
		if(!$this->isLoggedIn() && $this->_requestVars['userName'] !== null)
			$this->login();
		
		// IP Security check:
		if($this->isLoggedIn() && $this->_params['ipCheck']) {
			if(!isset($_SESSION[$this->_params['sessionValuePrefix'].'IP']) || $_SESSION[$this->_params['sessionValuePrefix'].'IP'] != $_SERVER['REMOTE_ADDR']) {
				// Logout message
				$text = '/logout '.$this->getUserName().' IP';
				$this->insertCustomMessage($this->_params['chatBotUserID'],$this->_params['chatBotUserName'],AJAX_CHAT_CHATBOT,$this->getChannel(),$text);
				$this->logout(false);
			}
		}
	}
		
	function login() {
		// Retrieve valid login user data (from request variables or session data):
		$userData = $this->getValidLoginUserData();
		
		if(!$userData) {
			$this->_infoMessages['errorInvalidUser'] = true;
			return false;
		}
		
		if(!$this->_params['allowGuestLogins'] && $userData['userRole'] == AJAX_CHAT_GUEST) {
			return false;
		}
		
		// Trim userName for use with the chat:
		$userData['userName'] = $this->trimUserName($userData['userName']);

		// Check if userName/userID is already in use:
		if($this->isUserInUse($userData['userID'], $userData['userName'])) {
			if($userData['userRole'] == AJAX_CHAT_USER || $userData['userRole'] == AJAX_CHAT_MODERATOR || $userData['userRole'] == AJAX_CHAT_ADMIN) {
				// Set the registered user inactive and remove the inactive users so the user can be logged in again:
				$this->setInactive($userData['userID']);
				$this->removeInactive();
			} else {
				$this->_infoMessages['errorUserInUse'] = true;
				return false;
			}
		}
		
		// Check if user is banned:
		if($this->isUserBanned($userData['userID'], $userData['userName'])) {
			$this->_infoMessages['errorBanned'] = true;
			return false;
		}

		// Regenerate session id:
		session_regenerate_id(true);

		// Log in:
		$this->setUserID($userData['userID']);
		$this->setUserName($userData['userName']);
		$this->setUserRole($userData['userRole']);
		$this->setLoggedIn(true);	
		$this->setLoginTimeStamp(time());
		
		// Check the given channelID, or get channelID from channelName:
		if($this->_requestVars['channelID'] === null) {
			$this->_requestVars['channelID'] = $this->getChannelIDFromChannelName($this->_requestVars['channelName']);		
		}
		if(!$this->validateChannel($this->_requestVars['channelID'])) {
			$this->_requestVars['channelID'] = $this->_params['defaultChannelID'];
		}

		$this->setChannel($this->_requestVars['channelID']);		
		$this->addToOnlineList();
			
		// IP Security check variable:
		$_SESSION[$this->_params['sessionValuePrefix'].'IP'] = $_SERVER['REMOTE_ADDR'];
			
		// Login message:
		$text = '/login '.$this->getUserName();
		$this->insertCustomMessage($this->_params['chatBotUserID'],$this->_params['chatBotUserName'],AJAX_CHAT_CHATBOT,$this->getChannel(),$text);

		// Create logs:
		if($this->_params['logsCreateLogs'] && $this->isTimeToCreateLogFiles())	
			$this->createLogFiles();

		return true;
	}
	
	function logout($insertLogoutMessage) {
		if($this->isLoggedIn()) {
			if($insertLogoutMessage) {
				// Logout message
				$text = '/logout '.$this->getUserName();
				$this->insertCustomMessage($this->_params['chatBotUserID'],$this->_params['chatBotUserName'],AJAX_CHAT_CHATBOT,$this->getChannel(),$text);
			}
			$this->removeFromOnlineList();
			$this->setLoggedIn(false);
			
			// Delete all session variables:
			$_SESSION = array();
			
			// Delete the session cookie:
			if(isset($_COOKIE[session_name()])) {
				setcookie(session_name(), '', time()-42000, '/');
			}
			
			// Destroy the session:
			@session_destroy();
		}
	}
	
	function switchChannel($channelName) {
		$channelID = $this->getChannelIDFromChannelName($channelName);		

		if($this->getChannel()===$channelID)
			return false;

		if(!$this->validateChannel($channelID)) {
			// Invalid channel:
			$text = '/error InvalidChannelName '.$channelName;
			$this->insertCustomMessage($this->_params['chatBotUserID'],$this->_params['chatBotUserName'],AJAX_CHAT_CHATBOT,$this->getUserID()+$this->_params['privateChannelDiff'],$text);
			return false;
		}

		// Channel leave message
		$text = '/channelLeave '.$this->getUserName();
		$this->insertCustomMessage($this->_params['chatBotUserID'],$this->_params['chatBotUserName'],AJAX_CHAT_CHATBOT,$this->getChannel(),$text);
			
		$this->setChannel($channelID);
		$this->updateOnlineList();

		// Channel enter message
		$text = '/channelEnter '.$this->getUserName();
		$this->insertCustomMessage($this->_params['chatBotUserID'],$this->_params['chatBotUserName'],AJAX_CHAT_CHATBOT,$this->getChannel(),$text);

		$this->_infoMessages['channelSwitch'] = $channelName;
		$this->_requestVars['lastID'] = 0;
	}
	
	function addToOnlineList() {
		$sql = 'INSERT INTO `'.$this->_params['dbTableNames']['online'].'`(
					userID,
					userName,
					userRole,
					channel,
					dateTime,
					ip
				)
				VALUES (
					'.$this->db->makeSafe($this->getUserID()).',
					'.$this->db->makeSafe($this->getUserName()).',
					'.$this->db->makeSafe($this->getUserRole()).',
					'.$this->db->makeSafe($this->getChannel()).',
					NOW(),
					INET_ATON("'.$_SERVER['REMOTE_ADDR'].'")
				);';	
		
		// Create a new SQLQuery:
		$result = new SQLQuery($sql, $this->db->getConnectionID());
		
		// Stop if an error occurs:
		if($result->error()) {
		    echo $result->getError();
		    die();
		}
	}
	
	function removeFromOnlineList() {
		$sql = 'DELETE FROM
					`'.$this->_params['dbTableNames']['online'].'`
				WHERE
					userID = '.$this->db->makeSafe($this->getUserID()).';';
		
		// Create a new SQLQuery:
		$result = new SQLQuery($sql, $this->db->getConnectionID());
		
		// Stop if an error occurs:
		if($result->error()) {
		    echo $result->getError();
		    die();
		}
	}
	
	function updateOnlineList() {
        $sql = 'UPDATE
					`'.$this->_params['dbTableNames']['online'].'`
				SET
					channel 	= '.$this->db->makeSafe($this->getChannel()).',
					dateTime 	= NOW(),
					ip			= INET_ATON("'.$_SERVER['REMOTE_ADDR'].'")
				WHERE
					userID = '.$this->db->makeSafe($this->getUserID()).';';
					
		// Create a new SQLQuery:
		$result = new SQLQuery($sql, $this->db->getConnectionID());
		
		// Stop if an error occurs:
		if($result->error()) {
		    echo $result->getError();
		    die();
		}
	}
	
	function initMessageHandling() {
		if(!$this->isLoggedIn())
			return false;
					
		if($this->_requestVars['text'])
			$this->insertMessage(substr($this->_requestVars['text'], 0, $this->_params['messageTextMaxLength']));
	}
	
	function getIDFromName($name) {
		$sql = 'SELECT
					userID
				FROM
					`'.$this->_params['dbTableNames']['online'].'`
				WHERE
					userName = '.$this->db->makeSafe($name).';';

		// Create a new SQLQuery:
		$result = new SQLQuery($sql, $this->db->getConnectionID());
		
		// Stop if an error occurs:
		if($result->error()) {
		    echo $result->getError();
		    die();
		}

		if($result->numRows() === 1) {
	        // Get the content:
	        $row = $result->fetch();
	        $result->free();
	        // Return the User ID:
	        return $row['userID'];
		} else {
			$result->free();
		}
		return -1;
	}

	function getIPFromID($userID) {
		$sql = 'SELECT
					INET_NTOA(ip) AS ip
				FROM
					`'.$this->_params['dbTableNames']['online'].'`
				WHERE
					userID = '.$this->db->makeSafe($userID).';';

		// Create a new SQLQuery:
		$result = new SQLQuery($sql, $this->db->getConnectionID());
		
		// Stop if an error occurs:
		if($result->error()) {
		    echo $result->getError();
		    die();
		}

		if($result->numRows() === 1) {
	        // Get the content:
	        $row = $result->fetch();
	        $result->free();
	        // Return the User ID:
	        return $row['ip'];
		} else {
			$result->free();
		}
		return null;
	}
	
	function getRoleFromID($userID) {
		$sql = 'SELECT
					userRole
				FROM
					`'.$this->_params['dbTableNames']['online'].'`
				WHERE
					userID = '.$this->db->makeSafe($userID).';';

		// Create a new SQLQuery:
		$result = new SQLQuery($sql, $this->db->getConnectionID());
		
		// Stop if an error occurs:
		if($result->error()) {
		    echo $result->getError();
		    die();
		}

		if($result->numRows() === 1) {
	        // Get the content:
	        $row = $result->fetch();
	        $result->free();
	        // Return the userRole:
	        return $row['userRole'];
		} else {
			$result->free();
		}
		return null;
	}

	function removeUnsafeCharacters($text) {
		// Replace NO-WS-CTL, non-whitespace control characters (RFC 2822), decimal 1–8, 11–12, 14–31, and 127:
		return preg_replace('/[\x0\x1\x2\x3\x4\x5\x6\x7\x8\xB\xC\xE\xF\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1A\x1B\x1C\x1D\x1E\x1F\x7F]/', '', $text);
	}
	
	function insertParsedMessage($text) {

		$userID = $this->getUserID();
		$userName = $this->getUserName();
		$userRole = $this->getUserRole();
		$channel = $this->getChannel();

		// If a queryUserName is set, sent all messages as private messages to this userName:
		if($this->getQueryUserName() && strpos($text, '/') !== 0)
			$text = '/msg '.$this->getQueryUserName().' '.$text;
		
		// Parse IRC-style commands:
		if(strpos($text, '/') === 0) {
			$textParts = explode(' ', $text);

			switch($textParts[0]) {
				
				// Channel switch:
				case '/join':
					if(count($textParts) > 1) {
						$this->switchChannel(implode(' ', array_slice($textParts, 1)));
						// Inserting info messages and updating online list is already done, return:
						return;
					} else {
						$userID = $this->_params['chatBotUserID'];
						$userName = $this->_params['chatBotUserName'];
						$userRole = AJAX_CHAT_CHATBOT;
						$channel = $this->getUserID() + $this->_params['privateChannelDiff'];
						$text = '/error MissingChannelName';
					}
					$this->insertCustomMessage($userID, $userName, $userRole, $channel, $text);
					return;
					
				// Logout:
				case '/quit':
				case '/leave':
				case '/part':
				case '/logout':
					$userID = $this->_params['chatBotUserID'];
					$userName = $this->_params['chatBotUserName'];
					$userRole = AJAX_CHAT_CHATBOT;
					$text = '/logout '.$this->getUserName();
					$this->logout(false);
					$this->insertCustomMessage($userID, $userName, $userRole, $channel, $text);
					return;
					
				// Private message:
				case '/msg':
				case '/privmsg':
				case '/describe':
					if(count($textParts) > 2) {
						// Get UserID from UserName:
						$toUserID = $this->getIDFromName($textParts[1]);
						if($toUserID == -1) {
							// Private UserNameNotFound message to current User:
							$userID = $this->_params['chatBotUserID'];
							$userName = $this->_params['chatBotUserName'];
							$userRole = AJAX_CHAT_CHATBOT;
							$channel = $this->getUserID() + $this->_params['privateChannelDiff'];
							$text = '/error UserNameNotFound '.$textParts[1];
						} else {
							// Insert own userName if /describe is used:
							$describe = ($textParts[0] == '/describe') ? $this->getUserName().' ' : '';
							
							// Copy of private message to current User:
							$text = '/privmsgto '.$textParts[1].' '.$describe.implode(' ', array_slice($textParts, 2));
							$channel = $this->getUserID() + $this->_params['privateChannelDiff'];
							$this->insertCustomMessage($userID, $userName, $userRole, $channel, $text);
							
							// Private message to requested User:
							$text = '/privmsg '.$describe.implode(' ', array_slice($textParts, 2));
							$channel = $toUserID + $this->_params['privateChannelDiff'];
						}
					} else {
						$userID = $this->_params['chatBotUserID'];
						$userName = $this->_params['chatBotUserName'];
						$userRole = AJAX_CHAT_CHATBOT;
						$channel = $this->getUserID() + $this->_params['privateChannelDiff'];
						if(count($textParts) == 2)
							$text = '/error MissingText';
						else
							$text = '/error MissingUserName';
					}
					$this->insertCustomMessage($userID, $userName, $userRole, $channel, $text);
					return;
				
				// Private channel:
				case '/query':
					// Message inserted will definitely be a chatBot message and sent to the own private channel:
					$userID = $this->_params['chatBotUserID'];
					$userName = $this->_params['chatBotUserName'];
					$userRole = AJAX_CHAT_CHATBOT;
					$channel = $this->getUserID() + $this->_params['privateChannelDiff'];
					if(count($textParts) == 1) {
						if($this->getQueryUserName()) {
							// Close the current query (private User-to-User channel):
							$text = '/queryClose '.$this->getQueryUserName();
							$this->setQueryUserName(null);
						} else {
							// Private NoOpenQuery message to current User:
							$text = '/error NoOpenQuery';
						}
					} else {
						if($this->getIDFromName($textParts[1]) == -1) {
							// Private UserNameNotFound message to current User:
							$text = '/error UserNameNotFound '.$textParts[1];
						} else {
							$this->setQueryUserName($textParts[1]);
							$text = '/queryOpen '.$textParts[1];
						}
					}
					$this->insertCustomMessage($userID, $userName, $userRole, $channel, $text);
					return;
				
				// Adding/removing users to an ignore list:
				case '/ignore':
					// Message inserted will definitely be a chatBot message and sent to the own private channel:
					$userID = $this->_params['chatBotUserID'];
					$userName = $this->_params['chatBotUserName'];
					$userRole = AJAX_CHAT_CHATBOT;
					$channel = $this->getUserID() + $this->_params['privateChannelDiff'];
					if(count($textParts) > 1) {
						if($this->getIgnoredUserNames() && in_array($textParts[1], $this->getIgnoredUserNames())) {
							$this->removeIgnoredUserName($textParts[1]);
							$text = '/ignoreRemoved '.$textParts[1];
						} else {
							$this->addIgnoredUserName($textParts[1]);
							$text = '/ignoreAdded '.$textParts[1];
						}
					} else {
						// Display a list of ignored users:
						if($this->getIgnoredUserNames() && count($this->getIgnoredUserNames()) > 0)
							$text = '/ignoreList '.join(' ', $this->getIgnoredUserNames());
						else
							$text = '/ignoreListEmpty -';
					}
					$this->insertCustomMessage($userID, $userName, $userRole, $channel, $text);
					return;
				
				// Kicking offending users from the chat:
				case '/kick':
					// Message inserted will definitely be a chatBot message:
					$userID = $this->_params['chatBotUserID'];
					$userName = $this->_params['chatBotUserName'];
					$userRole = AJAX_CHAT_CHATBOT;
					// Only moderators/admins may kick users:
					if($this->getUserRole() == AJAX_CHAT_ADMIN || $this->getUserRole() == AJAX_CHAT_MODERATOR) {
						if(count($textParts) > 1) {
							// Get UserID from UserName:
							$kickUserID = $this->getIDFromName($textParts[1]);
							if($kickUserID == -1) {
								// Private UserNameNotFound message to current User:
								$channel = $this->getUserID() + $this->_params['privateChannelDiff'];
								$text = '/error UserNameNotFound '.$textParts[1];
							} else {
								// Check the role of the user to kick:
								$kickUserRole = $this->getRoleFromID($kickUserID);
								if($kickUserRole == AJAX_CHAT_ADMIN || ($kickUserRole == AJAX_CHAT_MODERATOR && $this->getUserRole() != AJAX_CHAT_ADMIN)) {
									// Admins and moderators may not be kicked:
									$channel = $this->getUserID() + $this->_params['privateChannelDiff'];
									$text = '/error KickNotAllowed '.$textParts[1];
								} else {						
									// Kick User and insert message:
									$banMinutes = (count($textParts) > 2) ? $textParts[2] : null;
									$this->kickUser($kickUserID, $textParts[1], $banMinutes);
									$text = '/kick '.$textParts[1];
								}
							}
						} else {
							// Private MissingUserName message to current User:
							$channel = $this->getUserID() + $this->_params['privateChannelDiff'];
							$text = '/error MissingUserName';
						}
					} else {
						// Private CommandNotAllowed message to current User:
						$channel = $this->getUserID() + $this->_params['privateChannelDiff'];
						$text = '/error CommandNotAllowed '.$textParts[0];
					}
					$this->insertCustomMessage($userID, $userName, $userRole, $channel, $text);
					return;
				
				// Listing banned users:
				case '/bans':
					// Message inserted will definitely be a chatBot message:
					$userID = $this->_params['chatBotUserID'];
					$userName = $this->_params['chatBotUserName'];
					$userRole = AJAX_CHAT_CHATBOT;
					$channel = $this->getUserID() + $this->_params['privateChannelDiff'];
					// Only moderators/admins may ban users:
					if($this->getUserRole() == AJAX_CHAT_ADMIN || $this->getUserRole() == AJAX_CHAT_MODERATOR) {
						$this->removeExpiredBans();
						$bannedUsers = $this->getBannedUsers();
						if(count($bannedUsers) > 0) 
							$text = '/bans '.implode(' ', $bannedUsers);
						else
							$text = '/bansEmpty -';
					} else {
						// Private CommandNotAllowed message to current User:
						$text = '/error CommandNotAllowed '.$textParts[0];
					}				
					$this->insertCustomMessage($userID, $userName, $userRole, $channel, $text);
					return;
				
				// Unban user (remove from ban list):
				case '/unban':
					// Message inserted will definitely be a chatBot message:
					$userID = $this->_params['chatBotUserID'];
					$userName = $this->_params['chatBotUserName'];
					$userRole = AJAX_CHAT_CHATBOT;
					$channel = $this->getUserID() + $this->_params['privateChannelDiff'];
					// Only moderators/admins may unban users:
					if($this->getUserRole() == AJAX_CHAT_ADMIN || $this->getUserRole() == AJAX_CHAT_MODERATOR) {
						$this->removeExpiredBans();
						if(count($textParts) > 1) {
							if(!in_array($textParts[1], $this->getBannedUsers())) {
								// Private UserNameNotFound message to current User:
								$text = '/error UserNameNotFound '.$textParts[1];
							} else {
								// Unban User and insert message:
								$this->unbanUser($textParts[1]);
								$text = '/unban '.$textParts[1];	
							}
						} else {
							// Private MissingUserName message to current User:
							$text = '/error MissingUserName';
						}
					} else {
						// Private CommandNotAllowed message to current User:
						$text = '/error CommandNotAllowed '.$textParts[0];
					}
					$this->insertCustomMessage($userID, $userName, $userRole, $channel, $text);
					return;
				
				// Describing own actions:
				case '/me':
					if(count($textParts) > 1)
						$text = $this->getUserName().' '.implode(' ', array_slice($textParts, 1));
					else
						$text = $this->getUserName();
					// If we are in query mode (Private User-to-User channel), just insert modified text and return:
					if($this->getQueryUserName()) {
						$this->insertMessage($text);
						return;
					}
					$this->insertCustomMessage($userID, $userName, $userRole, $channel, $text);
					return;
				
				// Listing online Users:
				case '/who':
					$userID = $this->_params['chatBotUserID'];
					$userName = $this->_params['chatBotUserName'];
					$userRole = AJAX_CHAT_CHATBOT;
					$channel = $this->getUserID() + $this->_params['privateChannelDiff'];
					
					if(count($textParts) > 1) {
						$channelID = $this->getChannelIDFromChannelName($textParts[1]);
						if(!$this->validateChannel($channelID)) {
							// Invalid channel:
							$text = '/error InvalidChannelName '.$textParts[1];
						} else {
							// Get online users for the given channel:
							$onlineUsers = $this->getOnlineUsers(array($channelID));
							if(count($onlineUsers) > 0)
								$text = '/who '.implode(' ', $onlineUsers);
							else
								$text = '/whoEmpty -';
						}
					} else {
						// Get online users for all accessible channels:
						$text = '/who '.implode(' ', $this->getOnlineUsers($this->getChannels()));					
					}
					$this->insertCustomMessage($userID, $userName, $userRole, $channel, $text);
					return;
				
				// Listing available channels:
				case '/list':
					$userID = $this->_params['chatBotUserID'];
					$userName = $this->_params['chatBotUserName'];
					$userRole = AJAX_CHAT_CHATBOT;
					$channel = $this->getUserID() + $this->_params['privateChannelDiff'];
					$text = '/list '.implode(' ', array_flip($this->getChannels()));
					$this->insertCustomMessage($userID, $userName, $userRole, $channel, $text);
					return;
				
				// Listing information about a User:
				case '/whois':
					// Message inserted will definitely be a chatBot message:
					$userID = $this->_params['chatBotUserID'];
					$userName = $this->_params['chatBotUserName'];
					$userRole = AJAX_CHAT_CHATBOT;
					$channel = $this->getUserID() + $this->_params['privateChannelDiff'];
					// Only moderators/admins:
					if($this->getUserRole() == AJAX_CHAT_ADMIN || $this->getUserRole() == AJAX_CHAT_MODERATOR) {
						if(count($textParts) > 1) {
							// Get UserID from UserName:
							$whoisUserID = $this->getIDFromName($textParts[1]);
							if($whoisUserID == -1) {
								// Private UserNameNotFound message to current User:
								$text = '/error UserNameNotFound '.$textParts[1];
							} else {
								// List user information:
								$text = '/whois '.$textParts[1].' '.$this->getIPFromID($whoisUserID);
							}
						} else {
							// Private MissingUserName message to current User:
							$text = '/error MissingUserName';
						}
					} else {
						// Private CommandNotAllowed message to current User:
						$text = '/error CommandNotAllowed '.$textParts[0];
					}
					$this->insertCustomMessage($userID, $userName, $userRole, $channel, $text);
					return;
				
				// Rolling dice:
				case '/roll':
					// Message inserted will definitely be a chatBot message:
					$userID = $this->_params['chatBotUserID'];
					$userName = $this->_params['chatBotUserName'];
					$userRole = AJAX_CHAT_CHATBOT;
					
					if(count($textParts) > 1) {
						$diceParts = explode('d', $textParts[1]);
						if(count($diceParts) == 2) {
							$number = (int)$diceParts[0];
							$sides = (int)$diceParts[1];
							
							// Dice number must be an integer between 1 and 100, else roll only one:
							$number = ($number > 0 && $number <= 100) ?  $number : 1;
							
							// Sides must be an integer between 1 and 100, else take 6:
							$sides = ($sides > 0 && $sides <= 100) ?  $sides : 6;
							
							$text = '/roll '.$this->getUserName().' '.$number.'d'.$sides.' ';
							for($i=0; $i<$number; $i++) {
								if($i != 0)
									$text .= ',';
								$text .= $this->rollDice($sides);
							}
						} else {
							// if dice syntax is invalid, roll one d6:
							$text = '/roll '.$this->getUserName().' 1d6 '.$this->rollDice(6);
						}
					} else {
						// default is one d6:
						$text = '/roll '.$this->getUserName().' 1d6 '.$this->rollDice(6);
					}
					$this->insertCustomMessage($userID, $userName, $userRole, $channel, $text);
					return;					
				
				// Custom or unknown command:
				default:
					if($this->parseCustomCommands($text, $textParts))
						return;
					
					// Private UnknownCommand message to current User:
					$userID = $this->_params['chatBotUserID'];
					$userName = $this->_params['chatBotUserName'];
					$userRole = AJAX_CHAT_CHATBOT;
					$channel = $this->getUserID() + $this->_params['privateChannelDiff'];
					$text = '/error UnknownCommand '.$textParts[0];
					$this->insertCustomMessage($userID, $userName, $userRole, $channel, $text);
					return;	

			}

		}
		
		// No command found, just insert the plain message:
		$this->insertCustomMessage($userID, $userName, $userRole, $channel, $text);

	}
	
	function insertMessage($text) {
		$text = $this->removeUnsafeCharacters($text);
		
		if(trim($text) == '')
			return;
		
		$this->insertParsedMessage($text);
	}
	
	function insertCustomMessage($userID, $userName, $userRole, $channel, $text) {
		$sql = 'INSERT INTO `'.$this->_params['dbTableNames']['messages'].'`(
								userID,
								userName,
								userRole,
								channel,
								dateTime,
								ip,
								text
							)
				VALUES (
					'.$this->db->makeSafe($userID).',
					'.$this->db->makeSafe($userName).',
					'.$this->db->makeSafe($userRole).',
					'.$this->db->makeSafe($channel).',
					NOW(),
					INET_ATON("'.$_SERVER['REMOTE_ADDR'].'"),
					'.$this->db->makeSafe($text).'
				);';

		// Create a new SQLQuery:
		$result = new SQLQuery($sql, $this->db->getConnectionID());
		
		// Stop if an error occurs:
		if($result->error()) {
		    echo $result->getError();
		    die();
		}
	}
	
	function rollDice($sides) {
		// seed with microseconds since last "whole" second
		mt_srand((double)microtime()*1000000);
		
		return round(mt_rand(1, $sides));
	}
	
	function kickUser($userID, $userName, $banMinutes) {
		// Ban User for the given time in minutes:
		if($banMinutes)
			$this->banUser($userID, $userName, $banMinutes);

		// Remove given User from online list:
		$sql = 'DELETE FROM
					`'.$this->_params['dbTableNames']['online'].'`
				WHERE
					userID = '.$this->db->makeSafe($userID).';';
		
		// Create a new SQLQuery:
		$result = new SQLQuery($sql, $this->db->getConnectionID());
		
		// Stop if an error occurs:
		if($result->error()) {
		    echo $result->getError();
		    die();
		}
	}
	
	function getBannedUsers() {
		$sql = 'SELECT
					userName
				FROM
					`'.$this->_params['dbTableNames']['bans'].'`;';
		
		// Create a new SQLQuery:
		$result = new SQLQuery($sql, $this->db->getConnectionID());
		
		// Stop if an error occurs:
		if($result->error()) {
		    echo $result->getError();
		    die();
		}

		$bannedUsers = array();
		while($row = $result->fetch()) {
			array_push($bannedUsers, $row['userName']);
		}
		
		$result->free();
		
		return $bannedUsers;
	}
	
	function banUser($userID, $userName, $banMinutes) {
		// Remove expired bans:
		$this->removeExpiredBans();
		
		$ip = $this->getIPFromID($userID);
		if($ip) {
			$sql = 'INSERT INTO `'.$this->_params['dbTableNames']['bans'].'`(
						userID,
						userName,
						dateTime,
						ip
					)
					VALUES (
						'.$this->db->makeSafe($userID).',
						'.$this->db->makeSafe($userName).',
						DATE_ADD(NOW(), interval '.$this->db->makeSafe($banMinutes).' MINUTE),
						INET_ATON("'.$ip.'")
					);';	
			
			// Create a new SQLQuery:
			$result = new SQLQuery($sql, $this->db->getConnectionID());
			
			// Stop if an error occurs:
			if($result->error()) {
			    echo $result->getError();
			    die();
			}
		}
	}
	
	function unbanUser($userName) {
		$sql = 'DELETE FROM
					`'.$this->_params['dbTableNames']['bans'].'`
				WHERE
					userName = '.$this->db->makeSafe($userName).';';
		
		// Create a new SQLQuery:
		$result = new SQLQuery($sql, $this->db->getConnectionID());
		
		// Stop if an error occurs:
		if($result->error()) {
		    echo $result->getError();
		    die();
		}
	}
	
	function removeExpiredBans() {
		$sql = 'DELETE FROM
					`'.$this->_params['dbTableNames']['bans'].'`
				WHERE
					dateTime < NOW();';
		
		// Create a new SQLQuery:
		$result = new SQLQuery($sql, $this->db->getConnectionID());
		
		// Stop if an error occurs:
		if($result->error()) {
		    echo $result->getError();
		    die();
		}
	}
	
	function setInactive($userID) {
        $sql = 'UPDATE
					`'.$this->_params['dbTableNames']['online'].'`
				SET
					dateTime = DATE_SUB(NOW(), interval '.(intval($this->_params['inactiveTimeout'])+1).' MINUTE)
				WHERE
					userID = '.$this->db->makeSafe($userID).';';
					
		// Create a new SQLQuery:
		$result = new SQLQuery($sql, $this->db->getConnectionID());
		
		// Stop if an error occurs:
		if($result->error()) {
		    echo $result->getError();
		    die();
		}
	}
	
	function removeInactive() {
		$sql = 'SELECT
					userID,
					userName,
					channel
				FROM
					`'.$this->_params['dbTableNames']['online'].'`
				WHERE
					NOW() > DATE_ADD(dateTime, interval '.$this->_params['inactiveTimeout'].' MINUTE);';
		
		// Create a new SQLQuery:
		$result = new SQLQuery($sql, $this->db->getConnectionID());
		
		// Stop if an error occurs:
		if($result->error()) {
		    echo $result->getError();
		    die();
		}
		
		if($result->numRows() > 0) {
			$condition = '';
			while($row = $result->fetch()) {
				if(!empty($condition))
					$condition .= ' OR ';
				// Add userID to condition for removal:
				$condition .= 'userID='.$row['userID'];
				
				// Insert logout timeout message:
				$text = '/logout '.$row['userName'].' Timeout';
				$this->insertCustomMessage($this->_params['chatBotUserID'],$this->_params['chatBotUserName'],AJAX_CHAT_CHATBOT,$row['channel'],$text);
			}
			
			$result->free();
			
			$sql = 'DELETE FROM
						`'.$this->_params['dbTableNames']['online'].'`
					WHERE
						'.$condition.';';
			
			// Create a new SQLQuery:
			$result = new SQLQuery($sql, $this->db->getConnectionID());
			
			// Stop if an error occurs:
			if($result->error()) {
			    echo $result->getError();
			    die();
			}
		} else {
			$result->free();
		}
	}

	function updateOnlineStatus() {
		if($this->isLoggedIn()) {		
			// Update online status within the timeout span (timeout - 1 minute):
			if(!$this->getStatusUpdateTimeStamp() || ((time() - $this->getStatusUpdateTimeStamp()) > ($this->_params['inactiveTimeout']-1)*60)) {
				$this->updateOnlineList();
				$this->setStatusUpdateTimeStamp(time());
			}
		}
	}
	
	function checkAndRemoveInactive() {
		// Remove inactive users if logged out and every inactiveCheckInterval:
		if(!$this->getInactiveCheckTimeStamp() || ((time() - $this->getInactiveCheckTimeStamp()) > $this->_params['inactiveCheckInterval']*60)) {
			$this->removeInactive();
			$this->setInactiveCheckTimeStamp(time());
		}
	}
	
	function getXMLMessages() {
		$this->updateOnlineStatus();
		$this->checkAndRemoveInactive();
				
		$xml = '<?xml version="1.0" encoding="UTF-8"?>';
		$xml .= '<root>';
		
		if($this->isLoggedIn()) {			

			// Helper var to check if current user is still listed online:
			$isOnline = false;
			
			// Get the online users:
			$sql = 'SELECT
						userID,
						userName,
						userRole
					FROM
						`'.$this->_params['dbTableNames']['online'].'`
					WHERE
						channel = '.$this->getChannel().';';
			
				// Create a new SQLQuery:
			$result = new SQLQuery($sql, $this->db->getConnectionID());
			
			// Stop if an error occurs:
			if($result->error()) {
			    echo $result->getError();
			    die();
			}
			
			$users = '<users>';
			$messages = '';
			while($row = $result->fetch()) {
				// Check if current user is still listed online:
				if($row['userID'] == $this->getUserID())
					$isOnline = true;
	
				$users .= '<user';
				$users .= ' role="'.$row['userRole'].'"';
				$users .= '>';
				$users .= '<![CDATA['.strtr($row['userName'], $this->_specialChars).']]>';
				$users .= '</user>';
			}
			$users .= '</users>';
	
			$result->free();
			
			// Logout if current user is not listed online:
			if(!$isOnline)
				$this->logout(false);
		}
		
		if($this->isLoggedIn()) {

			// Go throug the info messages:
			foreach($this->_infoMessages as $key=>$value) {
				$xml .= '<info type="'.$key.'">';
				$xml .= '<![CDATA['.strtr($value, $this->_specialChars).']]>';
				$xml .= '</info>';
			}

			
			$xml .= $users;

			$ignoreUsers = '';
			if($this->getIgnoredUserNames())
				foreach($this->getIgnoredUserNames() as $userName)
					$ignoreUsers .= ' AND NOT (userName = "'.$userName.'")';

			// Get the last messages in descending order (this optimises the LIMIT usage):
			$sql = 'SELECT
						id,
						userName,
						userRole,
						UNIX_TIMESTAMP(dateTime) AS timeStamp,
						text
					FROM
						`'.$this->_params['dbTableNames']['messages'].'`
					WHERE
						id > '.$this->db->makeSafe($this->_requestVars['lastID']).'
						AND (
						channel = '.$this->getChannel().'
						OR
						channel = '.($this->getUserID()+$this->_params['privateChannelDiff']).'
						)
						AND
						NOW() < DATE_ADD(dateTime, interval '.$this->_params['oldMessagesTimeDiff'].')
						'.$ignoreUsers.'
					ORDER BY
						id
						DESC
					LIMIT '.$this->_params['oldMessagesLimit'].';';
						
			// Create a new SQLQuery:
			$result = new SQLQuery($sql, $this->db->getConnectionID());
			
			// Stop if an error occurs:
			if($result->error()) {
			    echo $result->getError();
			    die();
			}
			
			// Add the messages in reverse order so it is ascending again:
			$messages = '';
			while($row = $result->fetch()) {
				$message = '';
				$message .= '<message';
				$message .= ' id="'.$row['id'].'"';
				$message .= ' dateTime="'.date('r', $row['timeStamp']).'"';
				$message .= ' userRole="'.$row['userRole'].'"';
				$message .= '>';
				$message .= '<username><![CDATA['.strtr($row['userName'], $this->_specialChars).']]></username>';
				$message .= '<text><![CDATA['.strtr($row['text'], $this->_specialChars).']]></text>';
				$message .= '</message>';
				$messages = $message.$messages;
			}
			$messages = '<messages>'.$messages.'</messages>';
			
			$xml .= $messages;
			
			$result->free();
		} else {
			$xml .= '<info type="logout">';
			$xml .= '<![CDATA['.strtr($this->_params['logoutData'], $this->_specialChars).']]>';
			$xml .= '</info>';
		}
		
		$xml .= '</root>';
		
		return $xml;
	}
	
	function sendXMLMessages() {
		// Cache control headers:
		header('Cache-Control: no-cache, must-revalidate');
		// For Internet Explorer < 7:
		header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
		// Create a new Content-Type object:
		$contentType = new ContentType('UTF-8', 'text/xml');
		// Send content-type header:
		$contentType->send();
		echo $this->getXMLMessages();
	}
	
	function getXMLMessageLogs() {
		$sql = 'SELECT
					id,
					userID,
					userName,
					userRole,
					channel,
					UNIX_TIMESTAMP(dateTime) AS timeStamp,
					INET_NTOA(ip) AS ip,
					text
				FROM
					`'.$this->_params['dbTableNames']['messages'].'`
				ORDER BY
					id
				LIMIT '.$this->_params['logsMessageNumber'].';';
					
		// Create a new SQLQuery:
		$result = new SQLQuery($sql, $this->db->getConnectionID());
		
		// Stop if an error occurs:
		if($result->error()) {
		    echo $result->getError();
		    die();
		}
		
		$logs = '<?xml version="1.0" encoding="UTF-8"?>';
		$logs .= '<root>';
		while($row = $result->fetch()) {
			$logs .= '<message';
			$logs .= ' id="'.$row['id'].'"';
			$logs .= ' userID="'.$row['userID'].'"';
			$logs .= ' userRole="'.$row['userRole'].'"';
			$logs .= ' channel="'.$row['channel'].'"';
			$logs .= ' dateTime="'.date('r', $row['timeStamp']).'"';
			$logs .= ' ip="'.$row['ip'].'"';
			$logs .= '>';
			$logs .= '<username><![CDATA['.strtr($row['userName'], $this->_specialChars).']]></username>';
			$logs .= '<text><![CDATA['.strtr($row['text'], $this->_specialChars).']]></text>';
			$logs .= '</message>';
		}
		$logs .= '</root>';

		$result->free();
	
		return $logs;
	}
	
	function isTimeToCreateLogFiles() {
		$sql = 'SELECT
					COUNT(id) AS numRows
				FROM
					`'.$this->_params['dbTableNames']['messages'].'`
				WHERE
					dateTime < DATE_SUB(NOW(), interval '.$this->_params['oldMessagesTimeDiff'].');';

		// Create a new SQLQuery:
		$result = new SQLQuery($sql, $this->db->getConnectionID());
		
		// Stop if an error occurs:
		if($result->error()) {
		    echo $result->getError();
		    die();
		}
		// Return false if found:
	    $row = $result->fetch();
	    $result->free();
		if($row['numRows'] > $this->_params['logsMessageNumber'])
			return true;
		return false;
	}
	
	function createLogFiles() {
		// Filename is the current timestamp:
		$fileName = time();
		
		if($this->_params['logsUseGzipCompression']) {
			$handle = gzopen($this->_params['logsDirectory'].$fileName.'.xml.gz', "wb");
			gzwrite($handle, $this->getXMLMessageLogs());
			gzclose($handle);
		} else {	
			$handle = fopen($this->_params['logsDirectory'].$fileName.'.xml', "w");
			fwrite($handle, $this->getXMLMessageLogs());
			fclose($handle);
		}
		
		// Remove the logged messages from the database:
		$this->purgeLoggedMessages();
	}
	
	function purgeLoggedMessages() {
		$sql = 'DELETE FROM
					`'.$this->_params['dbTableNames']['messages'].'`
				ORDER BY
					id
				LIMIT '.$this->_params['logsMessageNumber'].';';
		
		// Create a new SQLQuery:
		$result = new SQLQuery($sql, $this->db->getConnectionID());
		
		// Stop if an error occurs:
		if($result->error()) {
		    echo $result->getError();
		    die();
		}
	}
	
	function getInfoMessages() {
		return $this->_infoMessages;
	}

	function getOnlineUsers($channelIDs) {
		if($channelIDs) {
			$condition = 'WHERE channel IN ('.implode(', ', $channelIDs).')';
		} else
			$condition = '';
		
		$sql = 'SELECT
					userName
				FROM
					`'.$this->_params['dbTableNames']['online'].'`
				'.$condition.';';
		
		// Create a new SQLQuery:
		$result = new SQLQuery($sql, $this->db->getConnectionID());
		
		// Stop if an error occurs:
		if($result->error()) {
		    echo $result->getError();
		    die();
		}
		
		$users = array();
		while($row = $result->fetch()) {
			array_push($users, $row['userName']);
		}
		
		$result->free();
		
		return $users;
	}
	
	function getQueryUserName() {
		if(isset($_SESSION[$this->_params['sessionValuePrefix'].'QueryUserName']))
			return $_SESSION[$this->_params['sessionValuePrefix'].'QueryUserName'];
		else
			return null;
	}
	
	function setQueryUserName($userName) {
		$_SESSION[$this->_params['sessionValuePrefix'].'QueryUserName'] = $userName;
	}
	
	function getIgnoredUserNames() {
		if(isset($_SESSION[$this->_params['sessionValuePrefix'].'IgnoredUserNames']))
			return $_SESSION[$this->_params['sessionValuePrefix'].'IgnoredUserNames'];
		else
			return null;
	}
	
	function resetIgnoredUserNames() {
		$_SESSION[$this->_params['sessionValuePrefix'].'IgnoredUserNames'] = null;
	}
	
	function addIgnoredUserName($userName) {
		if(!isset($_SESSION[$this->_params['sessionValuePrefix'].'IgnoredUserNames']))
			$_SESSION[$this->_params['sessionValuePrefix'].'IgnoredUserNames'] = array();
		array_push($_SESSION[$this->_params['sessionValuePrefix'].'IgnoredUserNames'], $userName);
	}
	
	function removeIgnoredUserName($userName) {
		if(isset($_SESSION[$this->_params['sessionValuePrefix'].'IgnoredUserNames'])) {
			// Anonymous function to filter out the given UserName:
			$filterFunction = create_function(
				'$value',
				'return $value !== "'.$userName.'";'
			);			
			$newArray = array_filter($_SESSION[$this->_params['sessionValuePrefix'].'IgnoredUserNames'], $filterFunction);
			// Save the new Array in the session:
			$_SESSION[$this->_params['sessionValuePrefix'].'IgnoredUserNames'] = $newArray;
		}
	}
	
	function getUserID() {
		if(isset($_SESSION[$this->_params['sessionValuePrefix'].'UserID']))
			return $_SESSION[$this->_params['sessionValuePrefix'].'UserID'];
		else
			return null;
	}
	
	function setUserID($id) {
		$_SESSION[$this->_params['sessionValuePrefix'].'UserID'] = $id;
	}

	function getUserName() {
		if(isset($_SESSION[$this->_params['sessionValuePrefix'].'UserName']))
			return $_SESSION[$this->_params['sessionValuePrefix'].'UserName'];
		else
			return null;
	}
	
	function setUserName($name) {
		$_SESSION[$this->_params['sessionValuePrefix'].'UserName'] = $name;
	}

	function trimUserName($userName) {
		$userName = $this->removeUnsafeCharacters($userName);
		
		$userName = $this->convertToUnicode($userName);

		// Replace any whitespace in the userName with the underscore "_":
		return substr(preg_replace('/\s/', '_', $userName), 0, $this->_params['userNameMaxLength']);
	}
	
	function convertToUnicode($str) {
		if($this->_params['contentEncoding'] != 'UTF-8') {
			if(function_exists('mb_convert_encoding')) {
				return mb_convert_encoding($str, 'UTF-8', mb_detect_encoding($str, array('UTF-8', $this->_params['contentEncoding'])));
			}
			if(function_exists('iconv')) {
				return iconv($this->_params['contentEncoding'], 'UTF-8', $str);
			}
		}
		return $str;
	}
	
	function convertFromUnicode($str) {
		if($this->_params['contentEncoding'] != 'UTF-8') {
			if(function_exists('mb_convert_encoding')) {
				return mb_convert_encoding($str, $this->_params['contentEncoding'], mb_detect_encoding($str, array('UTF-8', $this->_params['contentEncoding'])));
			}
			if(function_exists('iconv')) {
				return iconv('UTF-8', $this->_params['contentEncoding'], $str);
			}
		}
		return $str;
	}
	
	function getChannelIDFromChannelName($channelName) {
		// Remove slashes in front of apostrophes ('):
		$channelName = stripslashes($channelName);
		
		$channelName = $this->convertFromUnicode($channelName);
		
		$channels = $this->getChannels();
		if(array_key_exists($channelName,$channels))
			return $channels[$channelName];
		return null;
	}

	function getChannel() {
		if(isset($_SESSION[$this->_params['sessionValuePrefix'].'Channel']))
			return $_SESSION[$this->_params['sessionValuePrefix'].'Channel'];
		else
			return null;
	}
	
	function setChannel($channel) {
		$_SESSION[$this->_params['sessionValuePrefix'].'Channel'] = $channel;
	}

	function isLoggedIn() {
		if(isset($_SESSION[$this->_params['sessionValuePrefix'].'LoggedIn']))
			return $_SESSION[$this->_params['sessionValuePrefix'].'LoggedIn'];
		else
			return false;
	}
	
	function setLoggedIn($bool) {
		$_SESSION[$this->_params['sessionValuePrefix'].'LoggedIn'] = $bool;
	}
	
	function getLoginTimeStamp() {
		if(isset($_SESSION[$this->_params['sessionValuePrefix'].'LoginTimeStamp']))
			return $_SESSION[$this->_params['sessionValuePrefix'].'LoginTimeStamp'];
		else
			return null;
	}

	function setLoginTimeStamp($time) {
		$_SESSION[$this->_params['sessionValuePrefix'].'LoginTimeStamp'] = $time;
	}

	function getStatusUpdateTimeStamp() {
		if(isset($_SESSION[$this->_params['sessionValuePrefix'].'StatusUpdateTimeStamp']))
			return $_SESSION[$this->_params['sessionValuePrefix'].'StatusUpdateTimeStamp'];
		else
			return null;
	}

	function setStatusUpdateTimeStamp($time) {
		$_SESSION[$this->_params['sessionValuePrefix'].'StatusUpdateTimeStamp'] = $time;
	}

	function getInactiveCheckTimeStamp() {
		if(isset($_SESSION[$this->_params['sessionValuePrefix'].'InactiveCheckTimeStamp']))
			return $_SESSION[$this->_params['sessionValuePrefix'].'InactiveCheckTimeStamp'];
		else
			return null;
	}

	function setInactiveCheckTimeStamp($time) {
		$_SESSION[$this->_params['sessionValuePrefix'].'InactiveCheckTimeStamp'] = $time;
	}
	
	function setRequestUserID($userID) {
		$this->_requestVars['userID'] = $userID;
	}
	
	function setRequestUserName($userName) {
		$this->_requestVars['userName'] = $userName;
	}
	
	function setRequestChannelID($channelID) {
		$this->_requestVars['channelID'] = $channelID;
	}

	function setRequestChannelName($channelName) {
		$this->_requestVars['channelName'] = $channelName;
	}
	
	function getUserRole() {		
		if(isset($_SESSION[$this->_params['sessionValuePrefix'].'accessRole']))
			return $_SESSION[$this->_params['sessionValuePrefix'].'accessRole'];
		else
			return AJAX_CHAT_GUEST;
	}
	
	function setUserRole($role) {
		$_SESSION[$this->_params['sessionValuePrefix'].'accessRole'] = $role;
	}

	function isUserInUse($userID, $userName) {
		// Check if userID / userName already in use:			
		$sql = 'SELECT
					COUNT(userID) AS numRows
				FROM
					`'.$this->_params['dbTableNames']['online'].'`
				WHERE
					userID = '.$this->db->makeSafe($userID).'
				OR
					userName = '.$this->db->makeSafe($userName).';';

		// Create a new SQLQuery:
		$result = new SQLQuery($sql, $this->db->getConnectionID());
		
		// Stop if an error occurs:
		if($result->error()) {
		    echo $result->getError();
		    die();
		}
		// Return false if found:
	    $row = $result->fetch();
	    $result->free();
		if($row['numRows'] != 0)
			return true;
		return false;
	}
	
	function isUserBanned($userID, $userName) {
		// Check if userID, userName or ip is banned:			
		$sql = 'SELECT
					COUNT(userID) AS numRows
				FROM
					`'.$this->_params['dbTableNames']['bans'].'`
				WHERE
				(
					userID = '.$this->db->makeSafe($userID).'
				OR
					userName = '.$this->db->makeSafe($userName).'
				OR
					ip = INET_ATON("'.$_SERVER['REMOTE_ADDR'].'")
				)
				AND
					NOW() < dateTime;';

		// Create a new SQLQuery:
		$result = new SQLQuery($sql, $this->db->getConnectionID());
		
		// Stop if an error occurs:
		if($result->error()) {
		    echo $result->getError();
		    die();
		}
		// Return false if found:
	    $row = $result->fetch();
	    $result->free();
		if($row['numRows'] != 0)
			return true;
		return false;
	}
			
	function validateChannel($channelID) {
		if(in_array($channelID, $this->getChannels(), true))
			return true;
		return false;
	}
	
	function createGuestUserID() {
		// Use the current time() value minus privateChannelDiff as userID for guest logins:	
		return (time()-$this->_params['privateChannelDiff']);
	}
	
	// Override:
	// Returns an associative array containing userName, userID and userRole
	// Returns null if login is invalid
	function getValidLoginUserData() {

		// Don't allow empty or chatBot userName:
		if(empty($this->_requestVars['userName']) || $this->_requestVars['userName'] == $this->_params['chatBotUserName']) {
			return null;
		}

		$userData = array();
		$userData['userID'] = $this->createGuestUserID();
		$userData['userName'] = $this->_requestVars['userName'];
		$userData['userRole'] = AJAX_CHAT_GUEST;
		return $userData;
	}

	// Override:
	// Store the channels the current user has access to
	// Make sure channel names don't contain any whitespace
	function getChannels() {
		if($this->_channels === null) {
			$this->_channels = array();
			
			// Default channel, public to everyone:
			$this->_channels['Public'] = $this->_params['defaultChannelID'];
		}
		return $this->_channels;
	}

	// Override to add custom commands:
	// Return true if a custom command has been successfully parsed, else false
	// $text contains the whole message, $textParts the message split up as words array
	function parseCustomCommands($text, $textParts) {
		return false;
	}

}
?>