<?php
/*
 * Date:	03.09.2007
 *
 * File:	lib/class/CustomChat.php
 * Project:	PHP_AJAX_CHAT
 *
 * Author:	Sebastian Tschan
 * Website:	https://blueimp.net
 *
 * License:	GPL
 * 
 */

class CustomChat extends Chat {

	// Returns an associative array containing userName, userID and userRole
	// Returns null if login is invalid
	function getValidLoginUserData() {
		global $db, $auth, $user;
		
		// Check if we have a valid registered user:
		if(isset($user->data['is_registered']) && $user->data['is_registered'] == 1) {
			$userData = array();
			$userData['userID'] = $user->data['user_id'];
			// phpBB stores the ampersand ("&") as HTML entity ("&amp;") in usernames:
			$userData['userName'] = str_replace('&amp;', '&', $user->data['username']);
			
			if($auth->acl_get('a_'))
				$userData['userRole'] = AJAX_CHAT_ADMIN;
			elseif($auth->acl_get('m_'))
				$userData['userRole'] = AJAX_CHAT_MODERATOR;
			else
				$userData['userRole'] = AJAX_CHAT_USER;

			return $userData;
			
		} else {
			// Check if we have a valid guest user:
			
			// Don't allow empty or chatBot userName:
			if(empty($this->_requestVars['userName']) || $this->_requestVars['userName'] == $this->_params['chatBotUserName']) {
				return null;
			}
			
			// Don't allow a registered userName:
		
			// phpBB stores the ampersand ("&") as HTML entity ("&amp;") in usernames:
			$userName = str_replace('&', '&amp;', $this->_requestVars['userName']); 
			
			// Check if userName already exists in the phpBB users table:
			$sql = 'SELECT count(user_id) as numRows
						FROM '.USERS_TABLE.'
						WHERE
							username = "'.$db->sql_escape($userName).'";';
			$result = $db->sql_query($sql);
			$numRows = (int) $db->sql_fetchfield('numRows');
			$db->sql_freeresult($result);
			
			if($numRows === 0) {
				$userData = array();
				$userData['userID'] = $this->createGuestUserID();
				$userData['userName'] = $this->_requestVars['userName'];
				$userData['userRole'] = AJAX_CHAT_GUEST;
				return $userData;
			}

			return null;
		}
	}

	// Store the channels the current user has access to
	// Make sure channel names don't contain any whitespace
	function getChannels() {
		global $db, $auth;
		
		if($this->_channels === null) {
			$this->_channels = array();
			
			// Default channel, public to everyone:
			$this->_channels['Public'] = $this->_params['defaultChannelID'];

			// Get valid phpBB forums:
			$sql = 'SELECT forum_id, forum_name
						FROM '.FORUMS_TABLE.'
						WHERE
							forum_type=1
							AND
							forum_password="";';
			$result = $db->sql_query($sql);

			while ($row = $db->sql_fetchrow($result)) {
				if($auth->acl_get('f_read', $row['forum_id']))
					$this->_channels[preg_replace('/\s/', '_', $row['forum_name'])] = $row['forum_id'];
			}
			$db->sql_freeresult($result);
		}
		return $this->_channels;
	}

	// Method to set the style cookie depending on the phpBB user style:
	function setStyle() {
		global $config, $user, $db;
		
		if(isset($_COOKIE[$this->_params['sessionName'].'_style']) && in_array($_COOKIE[$this->_params['sessionName'].'_style'], $this->_params['styleAvailable']))
			return;
		
		$styleID = (!$config['override_user_style'] && $user->data['user_id'] != ANONYMOUS) ? $user->data['user_style'] : $config['default_style'];
		$sql = 'SELECT style_name
					FROM '.STYLES_TABLE.'
					WHERE
						style_id = "'.$db->sql_escape($styleID).'";';
		$result = $db->sql_query($sql);
		$styleName = $db->sql_fetchfield('style_name');
		$db->sql_freeresult($result);
		
		if(!in_array($styleName, $this->_params['styleAvailable']))
			$styleName = $this->_params['styleDefault'];
		
		setcookie($this->_params['sessionName'].'_style', $styleName, time()+60*60*24*356, '/');
		return;
	}

}
?>