<?php
/*
 * Date:	16.09.2007
 *
 * File:	lib/template/loggedIn.php
 * Project:	PHP_AJAX_CHAT
 *
 * Author:	Sebastian Tschan
 * Website:	https://blueimp.net
 *
 * License:	GPL
 * 
 */

// Create a new Content-Type object:
$contentType = new ContentType($params['contentEncoding'], null);
// Send content-type header:
$contentType->send();

if(stristr($contentType->get(),'xml'))
	echo '<?xml version="1.0" encoding="'.$params['contentEncoding'].'"?>'."\n";
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="<?php echo $language->getLangCode(); ?>" lang="<?php echo $language->getLangCode(); ?>">

<head>
	<meta http-equiv="Content-Type" content="<?php echo $contentType->get(); ?>" />
<?php
	foreach($params['styleAvailable'] as $style) {
		$alternate = ($style == $params['styleDefault']) ? '' : 'alternate ';
		echo '	<link rel="'.$alternate.'stylesheet" type="text/css" href="css/'.rawurlencode($style).'.css" title="'.htmlentities($style, ENT_QUOTES, $params['contentEncoding']).'"/>'."\n";
	}
?>
	<!--[if lt IE 7]>
		<link rel="stylesheet" type="text/css" href="css/ie5-6.css"/>
	<![endif]-->
	<script src="js/chat.js" type="text/javascript"></script>
	<script src="js/custom.js" type="text/javascript"></script>
	<script src="js/lang/<?php echo $language->getLangCode(); ?>.js" type="text/javascript"></script>
	<script src="js/config.js" type="text/javascript"></script>
	<script type="text/javascript">
		// <![CDATA[
			function displayOnlineUsers(checked) {
				if(document.getElementById('showHideHelpField').checked)
					document.getElementById('showHideHelpField').checked = false;
				ajaxChat.showHide('helpContainer', 'none');
				ajaxChat.showHide('onlineListContainer', null);
				if(checked)
					document.getElementById('chatList').style.right = '230px';
				else
					document.getElementById('chatList').style.right = '20px';
			}
			
			function displayHelp(checked) {
				if(document.getElementById('showHideOnlineField').checked)
					document.getElementById('showHideOnlineField').checked = false;
				ajaxChat.showHide('onlineListContainer', 'none');
				ajaxChat.showHide('helpContainer', null);
				if(checked)
					document.getElementById('chatList').style.right = '390px';
				else
					document.getElementById('chatList').style.right = '20px';
			}
			
			ajaxChat.init(ajaxChatConfig, ajaxChatLang, false);
			ajaxChat.initStyle();
		// ]]>
	</script>
	<title><?php echo htmlentities($lang['title'], ENT_QUOTES, $params['contentEncoding']); ?></title>
</head>

<body onload="ajaxChat.initialize();" onunload="ajaxChat.finalize();">
	<div id="content">
		<div id="headlineContainer">
			<h1><?php echo htmlentities($lang['title'], ENT_QUOTES, $params['contentEncoding']); ?></h1>
		</div>
		<form action="./" method="get" onsubmit="ajaxChat.logout(); return false;">
			<div id="logoutChannelContainer">
	  			<input type="submit" name="logout" id="logoutButton" value="<?php echo htmlentities($lang['logout'], ENT_QUOTES, $params['contentEncoding']); ?>"/>
				<label for="channelSelection"><?php echo htmlentities($lang['channel'], ENT_QUOTES, $params['contentEncoding']); ?>:</label>
				<select id="channelSelection" name="channelName" onchange="ajaxChat.switchChannel(this.options[this.selectedIndex].value)"><?php
					// Show a selection of the existing channels, preselect the current channel:
					foreach($chat->getChannels() as $key=>$value) {
						$selected = ($chat->getChannel() === $value) ? ' selected="selected"' : '';
						echo '<option value="'.htmlentities($key, ENT_QUOTES, $params['contentEncoding']).'"'.$selected.'>'.htmlentities($key, ENT_QUOTES, $params['contentEncoding']).'</option>';
					}
				?></select>
				<label for="styleSelection"><?php echo htmlentities($lang['style'], ENT_QUOTES, $params['contentEncoding']); ?>:</label>
				<select id="styleSelection" name="style" onchange="ajaxChat.setActiveStyleSheet(ajaxChat.getSelectedStyle());"><?php
					foreach($params['styleAvailable'] as $style) {
						$selected = ($style == $params['styleDefault']) ? ' selected="selected"' : '';
						echo '<option value="'.htmlentities($style, ENT_QUOTES, $params['contentEncoding']).'"'.$selected.'>'.htmlentities($style, ENT_QUOTES, $params['contentEncoding']).'</option>';
					}
				?></select>
	  		</div>
	  	</form>
		<div id="chatList"></div>
		<form action="./" method="post" enctype="application/x-www-form-urlencoded" onsubmit="ajaxChat.sendMessage(); return false;">
			<div id="messageContainer">
				<div id="messageTextContainer"><input id="messageText" type="text" name="text" maxlength="<?php echo $params['messageTextMaxLength']; ?>"/></div>
				<div id="submitButtonContainer"><input type="submit" name="submit" id="submitButton" value="<?php echo htmlentities($lang['messageSubmit'], ENT_QUOTES, $params['contentEncoding']); ?>"/></div>
			</div>
		</form>
		<div id="emoticonsContainer"></div>
		<div id="bbCodeContainer">
			<input type="button" value="<?php echo htmlentities($lang['bbCodeLabelBold'],		ENT_QUOTES, $params['contentEncoding']); ?>" title="<?php echo htmlentities($lang['bbCodeTitleBold'],		ENT_QUOTES, $params['contentEncoding']); ?>" onclick="ajaxChat.insertBBCode('b');" style="font-weight:bold;"/>
			<input type="button" value="<?php echo htmlentities($lang['bbCodeLabelItalic'],		ENT_QUOTES, $params['contentEncoding']); ?>" title="<?php echo htmlentities($lang['bbCodeTitleItalic'],		ENT_QUOTES, $params['contentEncoding']); ?>" onclick="ajaxChat.insertBBCode('i');" style="font-style:italic;"/>
			<input type="button" value="<?php echo htmlentities($lang['bbCodeLabelUnderline'],	ENT_QUOTES, $params['contentEncoding']); ?>" title="<?php echo htmlentities($lang['bbCodeTitleUnderline'],	ENT_QUOTES, $params['contentEncoding']); ?>" onclick="ajaxChat.insertBBCode('u');" style="text-decoration:underline;"/>
			<input type="button" value="<?php echo htmlentities($lang['bbCodeLabelQuote'],		ENT_QUOTES, $params['contentEncoding']); ?>" title="<?php echo htmlentities($lang['bbCodeTitleQuote'],		ENT_QUOTES, $params['contentEncoding']); ?>" onclick="ajaxChat.insertBBCode('quote');"/>
			<input type="button" value="<?php echo htmlentities($lang['bbCodeLabelCode'],		ENT_QUOTES, $params['contentEncoding']); ?>" title="<?php echo htmlentities($lang['bbCodeTitleCode'],		ENT_QUOTES, $params['contentEncoding']); ?>" onclick="ajaxChat.insertBBCode('code');"/>
			<input type="button" value="<?php echo htmlentities($lang['bbCodeLabelURL'],		ENT_QUOTES, $params['contentEncoding']); ?>" title="<?php echo htmlentities($lang['bbCodeTitleURL'],		ENT_QUOTES, $params['contentEncoding']); ?>" onclick="ajaxChat.insertBBCode('url');"/>
			<input type="button" value="<?php echo htmlentities($lang['bbCodeLabelColor'],		ENT_QUOTES, $params['contentEncoding']); ?>" title="<?php echo htmlentities($lang['bbCodeTitleColor'],		ENT_QUOTES, $params['contentEncoding']); ?>" onclick="ajaxChat.showHide('colorCodesContainer', null);"/>
		</div>
		<div id="colorCodesContainer" style="display:none;"></div>
		<div id="optionsContainer">
			<input type="checkbox" name="autoscroll" id="autoscrollField" checked="checked" onclick="ajaxChat.setAutoScroll(this.checked);"/>
			<label for="autoscrollField"><?php echo htmlentities($lang['autoScroll'], ENT_QUOTES, $params['contentEncoding']); ?></label>
			<input type="checkbox" name="showHideOnline" id="showHideOnlineField" checked="checked" onclick="displayOnlineUsers(this.checked);"/>
			<label for="showHideOnlineField"><?php echo htmlentities($lang['showOnlineUsers'], ENT_QUOTES, $params['contentEncoding']); ?></label>
			<input type="checkbox" name="showHideHelp" id="showHideHelpField" onclick="displayHelp(this.checked);"/>
			<label for="showHideHelpField"><?php echo htmlentities($lang['showHelp'], ENT_QUOTES, $params['contentEncoding']); ?></label>
  		</div>
		<div id="onlineListContainer">
			<h3><?php echo htmlentities($lang['onlineUsers'], ENT_QUOTES, $params['contentEncoding']); ?></h3>
	  		<div id="onlineList"></div>
	  	</div>
	  	<div id="helpContainer" style="display:none;">
 			<h3><?php echo htmlentities($lang['help'], ENT_QUOTES, $params['contentEncoding']); ?></h3>
 			<div id="helpList">
		  		<table><?php
		  			foreach($lang as $key=>$value)
		  				if(strpos($key, 'helpItemDesc') === 0)
		  					echo '<tr class="'.Functions::alternateRow().'"><td class="desc">'.htmlentities($value, ENT_QUOTES, $params['contentEncoding']).'</td>';
		  				elseif(strpos($key, 'helpItemCode') === 0)
		  					echo '<td class="code">'.htmlentities($value, ENT_QUOTES, $params['contentEncoding']).'</td></tr>';
		  		?></table>
	  		</div>
	  	</div>
	</div>
</body>

</html>